
#   Backlink counter
#   (c) 2014-2015 Scriptol.com
#   
#   Require one or more CSV file from Google Webmaster Tools.
#   The content must be a list of comma separated links each followed by two values.
#   
#   This scripts build the list of your HTML or PHP pages from a local copy,
#   get the number of backlinks from the CSV file, 
#   and build a table into an HTML file  to display the number of backlinks
#   for each of your pages.
#   
#   If a page is not in the CSV file, the number of BL is assumed to be zero.
#   
#   If several CSV files are provided, the difference between the first value
#   and the last value is displayed.  

include "path.sol"

# Change flags below to filter files

boolean ignoreImage = true       // assign false to include images
boolean pagesOnly = true         // assign false or change the list of extensions to add others
int minSize = 1024               // assign 0 to suppress the minimum size for a file

# End of flags section

array csv   // list of csv filenames   
int cellnum
file fh
text root
text prefix = ""
int rootlen
text ininame
text siteurl          // Domain 
text sitecount        // Backlinks to the home page   
int filecount = 0
int dircount = 0
array sums

# Change the lists below according to your site

array filesToExclude = [ "index.php", "header.php", "top.php", 
      "sidebar.php", "footer.php", "bottom.php" ]
      
array extToExclude = [".zip", ".gz", ".exe", ".xml", ".css", ".js", ".txt",
  ".lib" ]
  
array imgExtensions = [ ".jpg", ".png", ".gif", ".jpeg", ".bmp", ".ico" ]    

# Loading the ini file

void loadIni(text name)
  array ini
  if not file_exists(name) 
    text oldname = name
    name + ".ini"
    if not file_exists(name) let die("Ini file '$oldname' or $name' not found.")
  /if  
  ini.load(name)
  for text line in ini
    line = line.trim()
    if line.trim() = "" continue   
    if line[0 .. 3] = "site"
      root = line[5 ..]
      continue
    /if
    if line[0 .. 5] = "prefix"
      prefix = line[7 ..]
      continue
    /if    
    if line[0 .. 2] = "csv"
      csv.push(line[4 ..])
    /if
  /for
return        


# Put the csv in order by date. 

array sheets = []

# This CVS line split function is specialized to our problem
# Replace commas in quotes by dots, could be restored to commas at display

array splitCSVLine(text iline)
  int l = iline.length()
  boolean inquote = false
  for int i in 0 -- l
    if (iline[i] = '"') or (iline[i] = "'")
      inquote = not inquote
    /if
    if iline[i] = "," 
      if inquote ? iline[i] = '.'
    /if 
  /for     
return iline.split(",")

#  Load a CSV file, convert it into array, add to another array of arrays

void loadCSV()

  for text csvname in csv   // load each CSV file from the list at the top
    array rawcsv 
    if not file_exists(csvname) 
      die("Error, $csvname not found!")
    /if
    rawcsv.load(csvname)
    
    dict urlist    // internal array for a CSV
    for text row in rawcsv    // read each row in the CSV
      array rowcells = splitCSVLine(row)     // split a line
      text url = rowcells[0]
      url = url.trim()
      if url[0 .. 3] = 'http'
        siteurl = rowcells[0]
        if sitecount.length() > 0 ? sitecount + " "
        sitecount + rowcells[2].toText()
        continue          
      /if
      if url[0] = '/'
        text val = rowcells[2]    // only the second value (unique domain) is used
        urlist[url] = val    // add a line in the two-columns array
      /if
    /for
    urlist.sort()
    sheets.push(urlist)   // add a CSV as an array in the array of CSVs   
  /for
return    

# Make a list of CSV files from the prefix

void getCSVFiles(text p, text currdir)
   csv = []
   array filelist = scandir(currdir)
   int plen = p.length()
   text dummy, ext

   for text filename in filelist
     if filename = nil break
     if filename[0] = "." continue
     if filename in filesToExclude continue
     if Path.type(filename)  = "dir" continue   
     if filename[0 -- plen] = p
       dummy, ext = Path.splitExt(filename)
       if(ext.lower() != "csv") continue
       csv.push(filename)
     /if             
   /for
return

# Filter according to some extensions

boolean extFilter(text url)
  text ext = Path.getExtension(url)
  if ignoreImage and (ext.lower() in imgExtensions) return true
  if pagesOnly 
    if ext.lower() in extToExclude return true
  /if  
return false  


# Get number of backlinks for an URL, from a CSV 

int getCount(int sheet, text url)
  array s = sheets[sheet]
  if not array_key_exists(url, s) return 0
return s[url]  


# Add a line in the HTML file

void tohtml(text str)
  fh.write(str)
return  

# Add entry

void addEntry(text fullpath, boolean isdir)
     text relpath = fullpath[rootlen ..]
     if isdir
       tohtml("<tr class='dirhead'>")
       tohtml("<td><b><u>$relpath</u></b></td>")
     else  
       tohtml("<tr>")
       tohtml("<td>$relpath</td>")
     /if  
     int first
     int last
     for int cn in 0 -- cellnum
        //if isdir ? relpath + "/"
        int backlinks = getCount(cn, relpath)
        if cn = 0 ? first = backlinks
        if cn = cellnum -1 ? last = backlinks
        tohtml("<td>$backlinks</td>")
        sums[cn] + backlinks
     /for
     if cellnum > 1
      int diff = last - first
      text pad = ""
      if diff 
      > 0 : if diff < 10 let pad = "&nbsp;"
            tohtml("<td><b>$pad$diff</b></td>")
      < 0 : tohtml("<td>$diff</td>")
      else: tohtml("<td>&nbsp;</td>") 
      /if  
     /if
      tohtml("</tr>\n")     
return


# Read the content of a directory, recursively

void readFolder(text currdir)

   if not file_exists(currdir)
     print "Subdirectory '$currdir' not found."
     exit()
   /if

   if Path.type(currdir) <> "dir" ?  die(currdir + " not a directory")
 
   array filelist = scandir(currdir)
   array dirlist = []

   for text filename in filelist
     if filename = nil break
     if filename[0] = "." continue
     if filename in filesToExclude continue
              
     text fullpath = Path.merge(currdir, filename)     

     if Path.type(fullpath)
     = "dir" :              
         dirlist.push(fullpath + "/")
     = "file" : 
         if fileSize(fullpath) < minSize continue 
         if extFilter(filename) continue
         addEntry(fullpath, false)
         filecount + 1
     else
        tohtml("<unknow type> $fullpath")
     /if
   /for
   
   for text dirpath in dirlist
       if dirpath.trim() = "" continue 
       if dirpath[0] = "." continue
       addEntry(dirpath, true)
       dircount + 1
       readFolder(dirpath)
   /for

return

input "Type the name of an ini file or press Enter : ", ininame
if ininame = "" let ininame = "backcount.ini"

print "Loading configuration file", ininame
loadIni(ininame)

if not file_exists(root) let die("Directory '$root' not found.")
rootlen = root.length()

print "Site:", root
if prefix != ""
  print "CSV prefix:", prefix
  getCSVFiles(prefix, getcwd())
/if

int ctr = 0
for text c in csv 
  print ininame, "CSV:", c 
  sums[ctr] = 0
  ctr + 1
/for 
cellnum = csv.size()

if csv.size() = 0 
  die( "No CSV file found starting with $prefix" )
/if  

loadCSV()

text outname = csv[ csv.size() - 1]
int upos = outname.find("_")
outname = outname[0 .. upos + 8] + ".html"

fh = fopen(outname, "w")

tohtml("<!DOCTYPE html\n")
tohtml("<html>\n")
tohtml("<head>\n")
tohtml("<link type='text/css' href='backcount.css' rel='stylesheet'>\n")
tohtml("</head>\n")
tohtml("<body>\n")
tohtml("<h1>$siteurl  $sitecount</h1>\n")

tohtml("<table>\n<tr><td><b><u>Page</u></b>")
for int i in 1 .. csv.size()
  tohtml("</td><td><b><u>Links</u></b></td>")
/for
if csv.size() > 1 ? tohtml("<td><b><u>Diff</u></b></td>")  
tohtml("</tr>\n");

readFolder(root)

tohtml("</table>")    
tohtml("</div></body></html>")    

tohtml("<p>Total backlinks from different sites:&nbsp;")
for int i in 0 -- cellnum
  if i > 0 let tohtml("then &nbsp;")
  tohtml("<b>" + sums[i] + "</b> &nbsp;")
/for
tohtml("</p>")

tohtml("<p>$filecount files. $dircount dirs.</p>")
fh.close() 

print
echo  "Total backlinks from different sites: "
for int i in 0 -- cellnum
  echo sums[i], " "
/for
print
print "$outname created."  
